@@ -47,6 +47,7 @@ from mongoalchemy.query_expression import QueryField
 from mongoalchemy.fields import (ObjectIdField, Field, BadValueException, 
                                  SCALAR_MODIFIERS, DocumentField)
 from mongoalchemy.exceptions import DocumentException, MissingValueException, ExtraValueException, FieldNotRetrieved, BadFieldSpecification
+from mongoalchemy.util import resolve_name, FieldNotFoundException
 
 document_type_registry = defaultdict(dict)
 collection_registry = defaultdict(dict)
@@ -168,6 +169,12 @@ class Document(object):
         representations of the class
     '''
     
+    config_default_sort = None
+    ''' The default sort to use when querying.  If set, this sort will be 
+        applied to any query which a sort isn't used on. The format is the 
+        same as pymongo.  Example ``[('foo', 1), ('bar', -1)]``.
+    '''
+
     config_extra_fields = 'error'
     ''' Controls the method to use when dealing with fields passed in to the
         document constructor.  Possible values are 'error' and 'ignore'. Any 
@@ -216,7 +223,34 @@ class Document(object):
                     raise ExtraValueException(k)
 
         self.__extra_fields_orig = dict(self.__extra_fields)
+
+        # Validate defult sort
+        if self.config_default_sort:
+            for (name, direction) in self.config_default_sort:
+                try:
+                    resolve_name(type(self), name)
+                    dirs = (1, -1, pymongo.ASCENDING, pymongo.DESCENDING)
+                    if direction not in dirs:
+                        m = 'Bad sort direction on %s: %s' % (name, direction)
+                        raise BadFieldSpecification(m)
+                except FieldNotFoundException:
+                    raise BadFieldSpecification("Could not resolve field %s in" 
+                            " config_default_sort" % name)
+
     
+    @classmethod
+    def schema_json(cls):
+        ret = dict(fields={},
+                   config_namespace=cls.config_namespace,
+                   config_polymorphic=cls.config_polymorphic,
+                   config_polymorphic_collection=cls.config_polymorphic_collection,
+                   config_polymorphic_identity=cls.config_polymorphic_identity,
+                   config_full_name=cls.config_full_name,
+                   config_extra_fields=cls.config_extra_fields)
+        for f in cls.get_fields():
+            ret['fields'][f] = getattr(cls, f).schema_json()
+        return ret
+
     def __deepcopy__(self, memo):
         return type(self).unwrap(self.wrap(), session=self._get_session())
 
@@ -352,7 +386,16 @@ class Document(object):
             if isinstance(field, Index):
                 ret.append(field)
         return ret
-    
+    @classmethod
+    def transform_incoming(self, obj, session):
+        """ Tranform the SON object into one which will be able to be 
+            unwrapped by this document class.  
+
+            This method is designed for schema migration systems.
+        """
+        return obj
+
+
     @classmethod
     def __normalize(cls, fields):
         if not fields:
@@ -387,14 +430,15 @@ class Document(object):
         cls = self.__class__
         for name in self.get_fields():
             field = getattr(cls, name)
-            # if not isinstance(field, QueryField):
-            #     continue
             try:
                 value = getattr(self, name)
                 res[field.db_field] = field.wrap(value)
             except AttributeError, e:
                 if field.required:
                     raise MissingValueException(name)
+            except FieldNotRetrieved, fne:
+                if field.required:
+                    raise
         return res
         
     @classmethod
