@@ -7,10 +7,13 @@ class BadValueException(Exception):
         self.value = value
         self.cause = cause
         message = 'Bad value for field of type "%s".  Reason: "%s".' % (name, reason)
-        if cause != None:
+        if cause is not None:
             message = '%s Cause: %s' % (message, cause)
         Exception.__init__(self, message)
 
+class BadReferenceException(Exception):
+    pass
+
 class InvalidConfigException(Exception):
     ''' Raised when a bad value is passed in for a configuration that expects
         its values to obey certain constraints.'''
