@@ -61,6 +61,24 @@ class StringField(PrimitiveField):
         if self.min is not None and len(value) < self.min:
             self._fail_validation(value, 'Value too short (%d)' % len(value))
 
+class RegExStringField(PrimitiveField):
+    ''' Unicode Strings.  ``unicode`` is used to wrap and unwrap values,
+        and any subclass of basestring is an acceptable input, as long as
+        it matches the provided regex.'''
+    def __init__(self, regex, **kwargs):
+        ''' :param regex: instance of :class: `RegexObject` to match against
+            :param kwargs: arguments for :class:`Field`
+        '''
+        self.regex = regex
+        super(RegExStringField, self).__init__(constructor=unicode, **kwargs)
+
+    def validate_wrap(self, value):
+        ''' Validates the type and length of ``value`` '''
+        if not isinstance(value, basestring):
+            self._fail_validation_type(value, basestring)
+        if self.regex.match(value) is None:
+            self._fail_validation(value, 'Value does not match regular expression')
+
 class BinaryField(PrimitiveField):
     def __init__(self, **kwargs):
         super(BinaryField, self).__init__(constructor=Binary, **kwargs)
@@ -90,6 +108,11 @@ class NumberField(PrimitiveField):
         super(NumberField, self).__init__(constructor=constructor, **kwargs)
         self.min = min_value
         self.max = max_value
+    
+    def schema_json(self):
+        super_schema = super(NumberField, self).schema_json()
+        return dict(min_value=self.min, 
+                    max_value=self.max, **super_schema)
         
     def validate_wrap(self, value, *types):
         ''' Validates the type and value of ``value`` '''
@@ -150,7 +173,12 @@ class DateTimeField(PrimitiveField):
             import pytz
             self.utc = pytz.utc
             assert self.min is None and self.max is None
-    
+    def schema_json(self):
+        super_schema = super(DateTimeField, self).schema_json()
+        return dict(min_date=self.min, 
+                    max_date=self.max,
+                    use_tz=self.use_tz, **super_schema)
+
     def wrap(self, value):
         self.validate_wrap(value)
         value = self.constructor(value)
@@ -211,6 +239,11 @@ class TupleField(Field):
         self.size = len(item_types)
         self.types = item_types
     
+    def schema_json(self):
+        super_schema = super(TupleField, self).schema_json()
+        types = [t.schema_json() for t in self.types]
+        return dict(types=types, **super_schema)
+
     def set_parent_on_subtypes(self, parent):
         for type in self.types:
             type._set_parent(parent)
@@ -264,6 +297,10 @@ class GeoField(TupleField):
             :param kwargs: arguments for :class:`Field`
         '''
         super(GeoField, self).__init__(FloatField(), FloatField(), **kwargs)
+    def schema_json(self):
+        super_schema = super(GeoField, self).schema_json()
+        return dict(**super_schema)
+
 
 class EnumField(Field):
     ''' Represents a single value out of a list of possible values, all 
@@ -290,7 +327,12 @@ class EnumField(Field):
         #
         # for value in values:
         #     self.item_type.validate_wrap(value)
-    
+    def schema_json(self):
+        super_schema = super(EnumField, self).schema_json()
+        return dict(item_type=self.item_type.schema_json(),
+                    values=[self.item_type.wrap(v) for v in self.values], 
+                    **super_schema)
+
     def set_parent_on_subtypes(self, parent):
         self.item_type._set_parent(parent)
     
@@ -335,7 +377,9 @@ class AnythingField(Field):
         for free-form objects '''
     
     valid_modifiers = ANY_MODIFIER
-    
+    def schema_json(self):
+        return super(AnythingField, self).schema_json()
+
     def wrap(self, value):
         ''' Always returns the value passed in'''
         return value
@@ -358,16 +402,20 @@ class ObjectIdField(Field):
     valid_modifiers = SCALAR_MODIFIERS 
     
     def __init__(self, session=None, auto=False, **kwargs):
+        if auto:
+            kwargs['default_f'] = lambda : ObjectId()
         super(ObjectIdField, self).__init__(**kwargs)
-        self.auto = auto
+    def schema_json(self):
+        super_schema = super(ObjectIdField, self).schema_json()
+        return dict(auth=self.auto, **super_schema)
 
-    def set_default(self, value):
-        self._default = value
-    def get_default(self):
-        if self.auto:
-            self._default = ObjectId()
-        return self._default
-    default = property(get_default, set_default)
+    # def set_default(self, value):
+    #     super(ObjectIdField, self).set_default(value)
+    # def get_default(self):
+    #     if self.auto:
+    #         self.set_default(ObjectId())
+    #     return super(ObjectIdField, self).get_default()
+    # default = property(get_default, set_default)
 
     def gen(self):
         return ObjectId()
@@ -386,7 +434,7 @@ class ObjectIdField(Field):
         if len(value) != 24:
             self._fail_validation(value, 'hex object ID is the wrong length')
     
-    def wrap(self, value):
+    def wrap(self, value, session=None):
         ''' Validates that ``value`` is an ObjectId (or hex representation 
             of one), then returns it '''
         self.validate_wrap(value)
@@ -394,7 +442,7 @@ class ObjectIdField(Field):
             return ObjectId(value)
         return value
     
-    def unwrap(self, value):
+    def unwrap(self, value, session=None):
         ''' Validates that ``value`` is an ObjectId, then returns it '''
         self.validate_unwrap(value)
         return value
@@ -441,6 +489,13 @@ class ComputedField(Field):
         self.one_time = one_time
         self.__cached_value = UNSET
     
+    def schema_json(self):
+        super_schema = super(ComputedField, self).schema_json()
+        return dict(computed_type=self.computed_type.schema_json(),
+                    one_time=self.one_time,
+                    deps=list(self.deps), **super_schema)
+
+
     def __get__(self, instance, owner):
         # class method
         if instance is None:
