@@ -27,6 +27,7 @@ from copy import copy, deepcopy
 from mongoalchemy.exceptions import BadValueException
 from mongoalchemy.query_expression import QueryExpression, BadQueryException, flatten
 from mongoalchemy.update_expression import UpdateExpression, FindAndModifyExpression
+from mongoalchemy.util import resolve_name
 
 class BadResultException(Exception):
     ''' Only raised right now when .one() finds more than one object '''
@@ -47,7 +48,7 @@ class Query(object):
         self.session = session
         self.type = type
         self.__query = {}
-        self.sort = []
+        self._sort = []
         self._fields = None
         self.hints = []
         self._limit = None
@@ -56,15 +57,7 @@ class Query(object):
     
     def __iter__(self):
         return self.__get_query_result()
-    
-    def resolve_name(self, name):
-        if not isinstance(name, basestring) or name[0] == '$':
-            return name
-        ret = self.type
-        for part in name.split('.'):
-            ret = getattr(ret, part)
-        return ret
-    
+        
     @property
     def query(self):
         return flatten(self.__query)
@@ -108,7 +101,7 @@ class Query(object):
         '''
         qclone = Query(self.type, self.session)
         qclone.__query = deepcopy(self.__query)
-        qclone.sort = deepcopy(self.sort)
+        qclone._sort = deepcopy(self._sort)
         qclone._fields = deepcopy(self._fields)
         qclone._hints = deepcopy(self.hints)
         qclone._limit = deepcopy(self._limit)
@@ -157,7 +150,7 @@ class Query(object):
         return self.__hint(qfield, DESCENDING)
     
     def __hint(self, qfield, direction):
-        qfield = self.resolve_name(qfield)
+        qfield = resolve_name(self.type, qfield)
         name = str(qfield)
         for n, _ in self.hints:
             if n == name:
@@ -203,7 +196,7 @@ class Query(object):
         ''' Filter for the names in ``filters`` being equal to the associated 
             values.  Cannot be used for sub-objects since keys must be strings'''
         for name, value in filters.iteritems():
-            self.filter(self.resolve_name(name) == value)
+            self.filter(resolve_name(self.type, name) == value)
         return self
     
     def count(self, with_limit_and_skip=False):
@@ -225,7 +218,7 @@ class Query(object):
         if self._fields == None:
             self._fields = set()
         for f in fields:
-            f = self.resolve_name(f)
+            f = resolve_name(self.type, f)
             self._fields.add(f)
         self._fields.add(self.type.mongo_id)
         return self
@@ -237,7 +230,7 @@ class Query(object):
     def _apply_dict(self, qe_dict):
         ''' Apply a query expression, updating the query object '''
         for k, v in qe_dict.iteritems():
-            k = self.resolve_name(k)
+            k = resolve_name(self.type, k)
             if not k in self.__query:
                 self.__query[k] = v
                 continue
@@ -264,13 +257,29 @@ class Query(object):
         '''
         return self.__sort(qfield, DESCENDING)
     
+    def sort(self, *sort_tuples):
+        ''' pymongo-style sorting.  Accepts a list of tuples.
+
+            :param sort_tuples: varargs of sort tuples.
+        '''
+        query = self
+        for name, direction in sort_tuples:
+            field = resolve_name(self.type, name)
+            if direction in (ASCENDING, 1):
+                query = query.ascending(field)
+            elif direction in (DESCENDING, -1):
+                query = query.descending(field)
+            else:
+                raise BadQueryException('Bad sort direction: %s' % direction)
+        return query
+
     def __sort(self, qfield, direction):
-        qfield = self.resolve_name(qfield)
+        qfield = resolve_name(self.type, qfield)
         name = str(qfield)
-        for n, _ in self.sort:
+        for n, _ in self._sort:
             if n == name:
                 raise BadQueryException('Already sorting by %s' % name)
-        self.sort.append((name, direction))
+        self._sort.append((name, direction))
         return self
     
     def not_(self, *query_expressions):
@@ -307,7 +316,7 @@ class Query(object):
                 understands
         '''
         # TODO: make sure that this field represents a list
-        qfield = self.resolve_name(qfield)
+        qfield = resolve_name(self.type, qfield)
         self.filter(QueryExpression({ qfield : { '$in' : [qfield.wrap_value(value) for value in values]}}))
         return self
 
@@ -319,7 +328,7 @@ class Query(object):
                 understands
         '''
         # TODO: make sure that this field represents a list
-        qfield = self.resolve_name(qfield)
+        qfield = resolve_name(self.type, qfield)
         self.filter(QueryExpression({ qfield : { '$nin' : [qfield.wrap_value(value) for value in values]}}))
         return self
     
@@ -390,7 +399,7 @@ class QueryResult(object):
             obj = self.session.cache_read(value['_id'])
             if obj:
                 return obj
-            value = self.type.unwrap(value, fields=self.fields, session=self.session)
+            value = self.session._unwrap(self.type, value, fields=self.fields)
             if not isinstance(value, dict):
                 self.session.cache_write(value)
         return value
@@ -403,7 +412,7 @@ class QueryResult(object):
             obj = self.session.cache_read(value['_id'])
             if obj:
                 return obj
-            value = self.type.unwrap(value, session=self.session)
+            value = self.session._unwrap(self.type, value)
             self.session.cache_write(value)
             # value = self.session.localize(session, value)
         return value
