@@ -30,6 +30,7 @@ class BadQueryException(Exception):
 class FreeFormField(object):
     has_subfields = True
     no_real_attributes = True
+
     def __init__(self, name=None):
         self.__name = name
         self.db_field = name
@@ -48,6 +49,7 @@ class FreeFormField(object):
     __contains__ = is_valid_wrap
 
 class FreeFormDoc(object):
+    config_default_sort = None
     def __init__(self, name):
         self.__name = name
     def __getattr__(self, name):
@@ -57,6 +59,8 @@ class FreeFormDoc(object):
         return value
     def get_collection_name(self):
         return self.__name
+    def transform_incoming(self, obj, session):
+        return obj
     def get_indexes(self):
         return []        
     mongo_id = FreeFormField(name='_id')
